@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2004 Thorsten Kukuk
    The basis of this code is from the pam_ldap-148 package written
    by Luke Howard.
 
@@ -1921,29 +1921,6 @@ ldap_create_group (ldap_session_t *session, struct group *gr,
 	  gr->gr_name, baseou);
   asprintf (&groupdn, "cn=%s,%s", gr->gr_name, baseou);
 
-  /* create posixGroup object */
-  strvals[0][0] = "posixGroup";
-  strvals[0][1] = NULL;
-  strvals[1][0] = gr->gr_name;
-  strvals[1][1] = NULL;
-  asprintf (&strvals[2][0], "%u", gr->gr_gid);
-  strvals[2][1] = NULL;
-
-  mod[0].mod_values = strvals[0];
-  mod[0].mod_type = "objectClass";
-  mod[0].mod_op = LDAP_MOD_ADD;
-  mod[1].mod_values = strvals[1];
-  mod[1].mod_type = "cn";
-  mod[1].mod_op = LDAP_MOD_ADD;
-  mod[2].mod_values = strvals[2];
-  mod[2].mod_type = "gidNumber";
-  mod[2].mod_op = LDAP_MOD_ADD;
-
-  mods[0] = &mod[0];
-  mods[1] = &mod[1];
-  mods[2] = &mod[2];
-  mods[3] = NULL;
-
   if (!session->bind->bound_as_user)
     {
       rc = reopen_ldap_session (session);
@@ -1955,7 +1932,75 @@ ldap_create_group (ldap_session_t *session, struct group *gr,
 	return rc;
     }
 
+
+  /* create top objectClass.  */
+  strvals[0][0] = "namedObject";
+  strvals[0][1] = NULL;
+  strvals[1][0] = gr->gr_name;
+  strvals[1][1] = NULL;
+
+  mod[0].mod_values = strvals[0];
+  mod[0].mod_type = "objectClass";
+  mod[0].mod_op = LDAP_MOD_ADD;
+  mod[1].mod_values = strvals[1];
+  mod[1].mod_type = "cn";
+  mod[1].mod_op = LDAP_MOD_ADD;
+
+  mods[0] = &mod[0];
+  mods[1] = &mod[1];
+  mods[2] = NULL;
+
   rc = ldap_add_s (session->ld, groupdn, mods);
+  if (rc != 0)
+    {
+      ldap_delete_s (session->ld, groupdn);
+      free (groupdn);
+      return rc;
+    }
+
+#if 0
+  /* Create nameObject objectClass.  */
+  strvals[0][0] = "namedObject";
+  strvals[0][1] = NULL;
+
+  mod[0].mod_values = strvals[0];
+  mod[0].mod_type = "objectClass";
+  mod[0].mod_op = LDAP_MOD_ADD;
+
+  mods[0] = &mod[0];
+  mods[1] = NULL;
+
+  rc = ldap_modify_s (session->ld, groupdn, mods);
+  if (rc != 0)
+    {
+      ldap_delete_s (session->ld, groupdn);
+      free (groupdn);
+      return rc;
+    }
+#endif
+
+  strvals[0][0] = "posixGroup";
+  strvals[0][1] = NULL;
+  asprintf (&strvals[2][0], "%u", gr->gr_gid);
+  strvals[2][1] = NULL;
+
+  mod[0].mod_values = strvals[0];
+  mod[0].mod_type = "objectClass";
+  mod[0].mod_op = LDAP_MOD_ADD;
+  mod[1].mod_values = strvals[2];
+  mod[1].mod_type = "gidNumber";
+  mod[1].mod_op = LDAP_MOD_ADD;
+
+  mods[0] = &mod[0];
+  mods[1] = &mod[1];
+  mods[2] = NULL;
+
+  rc = ldap_modify_s (session->ld, groupdn, mods);
+
+  free (strvals[1][0]);
+
+  if (rc != 0)
+    ldap_delete_s (session->ld, groupdn);
 
   free (groupdn);
 
