@@ -1,5 +1,5 @@
-/* Copyright (C) 2002, 2003, 2004 Thorsten Kukuk
-   Author: Thorsten Kukuk <kukuk@suse.de>
+/* Copyright (C) 2002-2005 Thorsten Kukuk
+   Author: Thorsten Kukuk <kukuk@thkukuk.de>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
@@ -402,7 +402,7 @@ main (int argc, char *argv[])
 
   if (l_flag && !interactive)
     {
-      fprintf (stderr, _("%s: do not include \"l\" with other flags\n"),
+      fprintf (stderr, _("%s: Do not include \"l\" with other flags\n"),
 	       program);
       print_usage (stderr, program);
       return E_USAGE;
@@ -541,9 +541,9 @@ main (int argc, char *argv[])
 
       if (setgid (getgid ()) || setuid (getuid ()))
 	{
-	  syslog (LOG_ERR, "%s: failed to drop privileges: %s\n",
+	  syslog (LOG_ERR, "%s: Failed to drop privileges: %s\n",
 		  program, strerror (errno));
-	  fprintf (stderr, _("%s: failed to drop privileges: %s\n"),
+	  fprintf (stderr, _("%s: Failed to drop privileges: %s\n"),
 		   program, strerror (errno));
 	  return E_FAILURE;
         }
@@ -730,7 +730,7 @@ main (int argc, char *argv[])
 	printf (_("Aging information changed.\n"));
     }
 
-#if 0 /* XXX  logging */
+#if 0 /* XXX logging */
   if (pw_data->sp.sp_min != pw_data->spn.sp_min)
     LAUS_LOG(NO_TAG, "chage: password minimum age changed - user=%s, uid=%u, min=%li, oldmin=%li, by=%u",
 	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_min, pw_data->sp.sp_min, uid);
