@@ -85,7 +85,7 @@ print_help_shadow (const char *program)
   fputs (_("  -r room        Change your office room number\n"), stdout);
   fputs (_("  -w work_phone  Change your office phone number\n"), stdout);
   fputs (_("  -h home_phone  Change your home phone number\n"), stdout);
-  fputs (_("  -o other       Change the undefined portions of the GCOS field\n"), stdout);
+  fputs (_("  -o other       Change the undefined portions of the GECOS field\n"), stdout);
   fputs (_("  -q, --quiet    Don't be verbose\n"), stdout);
   if (strcmp(program, "chfn") == 0)
     fputs (_("  --service srv  Use nameservice 'srv'\n"), stdout);
@@ -121,7 +121,7 @@ print_help_util (const char *program)
   fputs (_("  -o office      Change your office room number\n"), stdout);
   fputs (_("  -p phone       Change your office phone number\n"), stdout);
   fputs (_("  -h home-phone  Change your home phone number\n"), stdout);
-  fputs (_("  -m other       Change the undefined portions of the GCOS field\n"), stdout);
+  fputs (_("  -m other       Change the undefined portions of the GECOS field\n"), stdout);
   if (strcmp(program, "chfn") == 0)
     fputs (_("  -r service     Use nameservice 'service'\n"), stdout);
   fputs (_("  -q, --quiet    Don't be verbose\n"), stdout);
