@@ -58,7 +58,7 @@
 static void
 print_usage (FILE * stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-g gid [-o] [-n new_name] group\n"), program);
+  fprintf (stream, _("Usage: %s [-g gid [-o]] [-n new_name] group\n"), program);
 }
 
 static void
