@@ -17,12 +17,116 @@
 #include <ctype.h>
 #include <netdb.h>
 #include <sys/socket.h>
+#include <sys/types.h>
+
+#ifdef HAVE_DIRENT_H
+#include <dirent.h>
+#endif
 
 #ifdef USE_SLP
 #include <slp.h>
 #endif
 
 
+#ifdef USE_GNUTLS
+static int
+start_request (HANDLE gnutls_session ssl, char *username, int admin_mode)
+{
+  request_header req;
+  char *locale = getenv ("LANG");
+  int ret;
+
+  if (admin_mode)
+    req.request = START_ADMIN;
+  else
+    req.request = START;
+  req.version = RPASSWD_VERSION;
+  req.data_len = strlen (username) + 1;
+  if (locale)
+    req.locale_len = strlen (locale) + 1;
+  else
+    req.locale_len = 0;
+
+  if ((ret = gnutls_record_send (ssl, &req, sizeof (request_header))) <= 0)
+    {
+      if (ret == 0)
+	PRINTF (ERR_HANDLE, _("error while sending start request: %s\n"),
+		_("Peer has closed the TLS connection"));
+      else
+	PRINTF (ERR_HANDLE, _("error while sending start request: %s\n"),
+		gnutls_strerror (ret));
+      return -1;
+    }
+
+  if (locale)
+    {
+      if ((ret = gnutls_record_send (ssl, locale, req.locale_len)) <= 0)
+	{
+	  if (ret == 0)
+	    PRINTF (ERR_HANDLE, _("error while sending locale data: %s\n"),
+		    _("Peer has closed the TLS connection"));
+	  else
+	    PRINTF (ERR_HANDLE, _("error while sending locale data: %s\n"),
+		    gnutls_strerror (ret));
+	  return -1;
+	}
+    }
+  if ((ret = gnutls_record_send (ssl, username, req.data_len)) <= 0)
+    {
+      if (ret == 0)
+	PRINTF (ERR_HANDLE, _("error while sending username: %s\n"),
+		_("Peer has closed the TLS connection"));
+      else
+	PRINTF (ERR_HANDLE, _("error while sending username: %s\n"),
+		gnutls_strerror (ret));
+      return -1;
+    }
+
+  return 0;
+}
+
+static int
+send_string (HANDLE gnutls_session ssl, u_int32_t retval, const char *str)
+{
+  int ret;
+  conv_header resp;
+
+  resp.retval = retval;
+  if (str == NULL)
+    resp.data_len = 0;
+  else
+    resp.data_len = strlen (str) + 1;
+
+  if ((ret = gnutls_record_send (ssl, &resp, sizeof (resp))) <= 0)
+    {
+      if (ret == 0)
+	PRINTF (ERR_HANDLE, _("error while sending string: %s\n"),
+		_("Peer has closed the TLS connection"));
+      else
+	PRINTF (ERR_HANDLE, _("error while sending string: %s\n"),
+		gnutls_strerror (ret));
+      return E_FAILURE;
+    }
+
+  if (str)
+    {
+      if ((ret = gnutls_record_send (ssl, str, resp.data_len)) <= 0)
+	{
+	  if (ret == 0)
+	    PRINTF (ERR_HANDLE, _("error while sending string: %s\n"),
+		    _("Peer has closed the TLS connection"));
+	  else
+	    PRINTF (ERR_HANDLE, _("error while sending string: %s\n"),
+		    gnutls_strerror (ret));
+	  return E_FAILURE;
+	}
+    }
+
+  return E_SUCCESS;
+}
+
+#else
+
 static int
 start_request (SSL *ssl, char *username, int admin_mode)
 {
@@ -75,11 +179,13 @@ send_string (SSL *ssl, u_int32_t ret, const char *str)
 
   return E_SUCCESS;
 }
+#endif
 
 #ifdef USE_SLP
 /* Ask SLP server for rpasswd service.  */
 
-struct slpcb {
+struct slpcb
+{
   char *srvurl;
   SLPError err;
   struct slpcb *next;
@@ -137,9 +243,9 @@ parse_slpcb (struct slpcb *cb)
 }
 
 static SLPBoolean
-MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
+MySLPSrvURLCallback (SLPHandle hslp __attribute__ ((unused)),
 		     const char *srvurl,
-		     unsigned short lifetime __attribute__((unused)),
+		     unsigned short lifetime __attribute__ ((unused)),
 		     SLPError errcode, void *cookie)
 {
   struct slpcb *cb = (struct slpcb *) cookie;
@@ -147,10 +253,10 @@ MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
   if (errcode == SLP_OK)
     {
       if (cb->srvurl != NULL)
-        {
-          struct slpcb *cbt = malloc (sizeof (struct slpcb));
-          if (cbt == NULL)
-            return SLP_FALSE;
+	{
+	  struct slpcb *cbt = malloc (sizeof (struct slpcb));
+	  if (cbt == NULL)
+	    return SLP_FALSE;
 
 	  cbt->srvurl = cb->srvurl;
 	  cbt->hostp = cb->hostp;
@@ -159,7 +265,7 @@ MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
 	  cbt->next = cb->next;
 	  cb->next = cbt;
 	  cb->descr = NULL;
-        }
+	}
       cb->srvurl = strdup (srvurl);
       parse_slpcb (cb);
       cb->err = SLP_OK;
@@ -168,14 +274,14 @@ MySLPSrvURLCallback (SLPHandle hslp __attribute__((unused)),
   else if (errcode != SLP_LAST_CALL)
     cb->err = errcode;
 
-  return SLP_FALSE; /* We don't wan't to be called again.  */
+  return SLP_FALSE;		/* We don't wan't to be called again.  */
 }
 
 static SLPBoolean
-MySLPAttrCallback (SLPHandle hslp __attribute__((unused)),
-		   const char *attrlist, SLPError errcode, void *cookie )
+MySLPAttrCallback (SLPHandle hslp __attribute__ ((unused)),
+		   const char *attrlist, SLPError errcode, void *cookie)
 {
-  char **descr = (char **)cookie;
+  char **descr = (char **) cookie;
 
   if (errcode == SLP_OK)
     {
@@ -184,7 +290,7 @@ MySLPAttrCallback (SLPHandle hslp __attribute__((unused)),
       if (cp == NULL)
 	return SLP_FALSE;
 
-      *descr = strdup (cp+13);
+      *descr = strdup (cp + 13);
 
       cp = strchr (*descr, ')');
       if (cp != NULL)
@@ -197,7 +303,7 @@ MySLPAttrCallback (SLPHandle hslp __attribute__((unused)),
 static int
 query_slp (HANDLE char **hostp, char **portp, char **descrp)
 {
-  struct slpcb *cb, callbackres = {NULL, 0, NULL, NULL, NULL, NULL};
+  struct slpcb *cb, callbackres = { NULL, 0, NULL, NULL, NULL, NULL };
   SLPError err;
   SLPHandle hslp;
 
@@ -232,14 +338,14 @@ query_slp (HANDLE char **hostp, char **portp, char **descrp)
       char *buf;
 
       asprintf (&buf, "service:rpasswdd://%s:%s/", cb->hostp,
-		cb->portp?:"774");
-      err = SLPFindAttrs (hslp, buf, "",
-			  "", /* use configured scopes */
+		cb->portp ? : "774");
+      err = SLPFindAttrs (hslp, buf, "", "",	/* use configured scopes */
 			  MySLPAttrCallback, &(cb->descr));
       free (buf);
       if (err != SLP_OK)
 	{
-	  PRINTF (STD_HANDLE, _("Error while searching for SLP description.\n"));
+	  PRINTF (STD_HANDLE,
+		  _("Error while searching for SLP description.\n"));
 	  return -1;
 	}
       cb = cb->next;
@@ -250,12 +356,12 @@ query_slp (HANDLE char **hostp, char **portp, char **descrp)
 
   cb = &callbackres;
 #if defined(SELECT_SRVURL)
-  if (cb->next != NULL) /* Only if we have more than one entry.  */
+  if (cb->next != NULL)		/* Only if we have more than one entry.  */
     {
       /* Ask the user which one to use.  */
       struct slpcb *tcb = &callbackres;
-      int choice = 0;     /* index of the user's choice.  */
-      char response[20];  /* string to hold the user's response.  */
+      int choice = 0;		/* index of the user's choice.  */
+      char response[20];	/* string to hold the user's response.  */
       int i = 0;
 
       printf (_("\nPlease select a server:\n"));
@@ -320,12 +426,11 @@ parse_reqcert (const char *str)
 
 /* Load the config file (/etc/rpasswd.conf)  */
 static int
-load_config (const char *configfile, char **hostp, char **portp,
-	     int *reqcertp
+load_config (const char *configfile, char **hostp, char **portp, int *reqcertp
 #ifdef DO_VERBOSE_OUTPUT
-	     ,int verbose, int check_syntax
+	     , int verbose, int check_syntax
 #endif
-	     )
+  )
 {
   FILE *fp;
   char *buf = NULL;
@@ -430,7 +535,7 @@ load_config (const char *configfile, char **hostp, char **portp,
 	  ++bad_entries;
 	}
       else
-        PRINTF (ERR_HANDLE, _("Entry \"%s\" is not valid, ignored!\n"), cp);
+	PRINTF (ERR_HANDLE, _("Entry \"%s\" is not valid, ignored!\n"), cp);
 #endif
     }
   fclose (fp);
@@ -467,8 +572,7 @@ load_config (const char *configfile, char **hostp, char **portp,
 }
 
 static int
-connect_to_server (HANDLE const char *hostp, const char *portp,
-		   int family)
+connect_to_server (HANDLE const char *hostp, const char *portp, int family)
 {
 #ifdef NI_WITHSCOPEID
   const int niflags = NI_NUMERICHOST | NI_WITHSCOPEID;
@@ -491,7 +595,8 @@ connect_to_server (HANDLE const char *hostp, const char *portp,
       if (error == EAI_NONAME)
 	{
 	  PRINTF (ERR_HANDLE,
-		  _("Hostname or service not known for specified protocol\n"));
+		  _
+		  ("Hostname or service not known for specified protocol\n"));
 	  return -1;
 	}
       else if (error == EAI_SERVICE)
@@ -556,7 +661,7 @@ connect_to_server (HANDLE const char *hostp, const char *portp,
 			   hbuf, sizeof (hbuf), NULL, 0, niflags) != 0)
 	    strcpy (hbuf, "(invalid)");
 	  PRINTF (ERR_HANDLE, _("connect to address %s: %s\n"), hbuf,
-		   strerror (errno));
+		  strerror (errno));
 	  close (sock);
 	  sock = -1;
 	  continue;
@@ -568,9 +673,253 @@ connect_to_server (HANDLE const char *hostp, const char *portp,
   return sock;
 }
 
+#ifdef USE_GNUTLS
+
+#include <gnutls/x509.h>
+
+#ifndef HAVE_GNUTLS_PK_ALGORITHM_GET_NAME
+
+static const char *
+gnutls_pk_algorithm_get_name (gnutls_pk_algorithm algorithm)
+{
+  if (algorithm == GNUTLS_PK_RSA)
+    return "RSA";
+  else if (algorithm == GNUTLS_PK_DSA)
+    return "DSA";
+  else
+    return "UNKNOWN";
+}
+#endif
+
+/* This function will print information about this session's peer
+ * certificate.
+ */
+static void
+print_x509_certificate_info (HANDLE gnutls_session session)
+{
+  char dn[128];
+  size_t size;
+  unsigned int algo, bits;
+  time_t expiration_time, activation_time;
+  const gnutls_datum *cert_list;
+  int cert_list_size = 0;
+  gnutls_x509_crt cert;
+
+  /* This function only works for X.509 certificates.
+   */
+  if (gnutls_certificate_type_get (session) != GNUTLS_CRT_X509)
+    return;
+
+  cert_list = gnutls_certificate_get_peers (session, &cert_list_size);
+  if (cert_list_size <= 0)
+    return;
+
+  PRINTF (STD_HANDLE, _("Server certificate info:\n"));
+
+  /* we only print information about the first certificate. */
+  gnutls_x509_crt_init (&cert);
+
+  gnutls_x509_crt_import (cert, &cert_list[0], GNUTLS_X509_FMT_DER);
+
+  expiration_time = gnutls_x509_crt_get_expiration_time (cert);
+  activation_time = gnutls_x509_crt_get_activation_time (cert);
+
+  PRINTF (STD_HANDLE, _("  Certificate is valid since: %s"),
+			ctime (&activation_time));
+  PRINTF (STD_HANDLE, _("  Certificate expires: %s"),
+	  ctime (&expiration_time));
+
+  /* Extract some of the public key algorithm's parameters
+   */
+  algo = gnutls_x509_crt_get_pk_algorithm (cert, &bits);
+  PRINTF (STD_HANDLE, _("  Certificate public key: %s"),
+	  gnutls_pk_algorithm_get_name (algo));
+
+  /* Print the version of the X.509
+   * certificate.
+   */
+  PRINTF (STD_HANDLE, _("  Certificate version: #%d\n"),
+	  gnutls_x509_crt_get_version (cert));
+
+  size = sizeof (dn);
+  gnutls_x509_crt_get_dn (cert, dn, &size);
+  PRINTF (STD_HANDLE, _("  DN: %s\n"), dn);
+
+  size = sizeof (dn);
+  gnutls_x509_crt_get_issuer_dn (cert, dn, &size);
+  PRINTF (STD_HANDLE, _("  Issuer's DN: %s\n"), dn);
+
+  PRINTF (STD_HANDLE, "\n");
+
+  gnutls_x509_crt_deinit (cert);
+}
+
+
+static int
+start_ssl (HANDLE long sock, int reqcert, int verbose, gnutls_session session,
+	   gnutls_certificate_credentials *cred)
+{
+  gnutls_certificate_credentials xcred;
+  DIR *dir = opendir ("/etc/ssl/certs");
+  struct dirent *entry;
+  int ret;
+
+  /* Allow connections to servers that have OpenPGP keys as well. */
+  const int cert_type_priority[3] =
+    { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
+
+
+  /* X509 stuff */
+  gnutls_certificate_allocate_credentials (&xcred);
+
+  /* sets the trusted cas files from /etc/ssl/certs and try to
+     add /etc/rpasswdd.pem (default from server). */
+  if (dir != NULL)
+    {
+      while ((entry = readdir (dir)) != NULL)
+	{
+	  /* Skip "." and ".." directory entries.  */
+	  if (strcmp (entry->d_name, ".") == 0 ||
+	      strcmp (entry->d_name, "..") == 0)
+	    continue;
+	  else
+	    {
+	      char srcfile[strlen ("/etc/ssl/certs") +
+			   strlen (entry->d_name) + 2];
+	      struct stat st;
+	      char *cp;
+
+	      /* create source and destination filename with full path.  */
+	      cp = stpcpy (srcfile, "/etc/ssl/certs");
+	      *cp++ = '/';
+	      strcpy (cp, entry->d_name);
+
+	      if (lstat (srcfile, &st) != 0)
+		continue;
+
+	      if (!S_ISLNK(st.st_mode) && !S_ISDIR(st.st_mode) &&
+		  !S_ISCHR(st.st_mode) && !S_ISBLK(st.st_mode))
+		{
+		  /* XXX error handling! */
+		  gnutls_certificate_set_x509_trust_file (xcred, srcfile,
+							  GNUTLS_X509_FMT_PEM);
+
+		}
+	    }
+	}
+      closedir (dir);
+    }
+  gnutls_certificate_set_x509_trust_file (xcred, "/etc/rpasswdd.pem",
+					  GNUTLS_X509_FMT_PEM);
+
+  *cred = xcred;
+
+  /* Use default priorities */
+  gnutls_set_default_priority (session);
+  gnutls_certificate_type_set_priority (session, cert_type_priority);
+
+  /* put the x509 credentials to the current session.  */
+  gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, xcred);
+
+  gnutls_transport_set_ptr (session, (gnutls_transport_ptr) sock);
+
+  /* Perform the TLS handshake.  */
+  ret = gnutls_handshake (session);
+
+  if (ret < 0)
+    {
+      PRINTF (ERR_HANDLE, _("Handshake failed: %s\n"),
+	      gnutls_strerror (ret));
+      return E_SSL_FAILURE;
+    }
+
+  if (reqcert > 0 || verbose)
+    {
+      gnutls_kx_algorithm kx;
+      unsigned int verify_result = 0;
+      time_t now;
+
+      /* print the key exchange's algorithm name.  */
+      kx = gnutls_kx_get (session);
+
+#ifdef HAVE_GNUTLS_CERTIFICATE_VERIFY_PEERS2
+      ret = gnutls_certificate_verify_peers2 (session, &verify_result);
+#else
+      ret = gnutls_certificate_verify_peers (session);
+      if (ret > 0)
+	verify_result = ret;
+#endif
+      if (ret < 0)
+	{
+	  PRINTF (ERR_HANDLE, _("TLS certificate error: %s\n"),
+		  gnutls_strerror (verify_result));
+	  return E_SSL_FAILURE;
+	}
+
+      /* Following two steps are optional and not required for
+         data exchange to be successful except the client couldn't verfiy
+         the server certificate.  */
+      if (verify_result || verbose)
+	{
+	  /* Get the cipher.  */
+	  PRINTF (STD_HANDLE, _("%s connection using %s-%s (%s)\n\n"),
+		  gnutls_protocol_get_name (gnutls_protocol_get_version
+					    (session)),
+		  gnutls_cipher_get_name (gnutls_cipher_get (session)),
+		  gnutls_mac_get_name (gnutls_mac_get (session)),
+		  gnutls_certificate_type_get_name
+		  (gnutls_certificate_type_get (session)));
+
+#if defined(IAM_PAM_MODULE)
+	  print_x509_certificate_info (pamh, session);
+#else
+	  print_x509_certificate_info (session);
+#endif
+
+	  if (verify_result & GNUTLS_CERT_SIGNER_NOT_FOUND)
+	    {
+	      PRINTF (ERR_HANDLE,
+		      _
+		      ("TLS authentication error: server certificate issuer is unknown.\n"));
+	      if (reqcert >= 2)
+		return E_SSL_FAILURE;
+	    }
+	  else if (verify_result & GNUTLS_CERT_INVALID)
+	    {
+	      PRINTF (ERR_HANDLE,
+		      _
+		      ("TLS authentication error: server certificate is NOT trusted.\n"));
+	      if (reqcert >= 2)
+		return E_SSL_FAILURE;
+	    }
+	}
+
+      now = time (NULL);
+
+      if (gnutls_certificate_activation_time_peers (session) > now)
+	{
+	  PRINTF (ERR_HANDLE,
+		  _("TLS authentication error: server certificate not yet activated.\n"));
+	  if (reqcert >= 2)
+	    return E_SSL_FAILURE;
+	}
+
+      if (gnutls_certificate_expiration_time_peers (session) < now)
+	{
+	  PRINTF (ERR_HANDLE,
+		  _("TLS authentication error: server certificate expired.\n"));
+	  if (reqcert >= 2)
+	    return E_SSL_FAILURE;
+	}
+    }
+
+  return 0;
+}
+
+#else
 static int
 start_ssl (HANDLE int sock, int reqcert, int verbose,
-	   SSL_CTX **ctx, SSL **ssl)
+	   SSL_CTX ** ctx, SSL ** ssl)
 {
   X509 *server_cert;
   char *str;
@@ -594,7 +943,7 @@ start_ssl (HANDLE int sock, int reqcert, int verbose,
   if (!SSL_CTX_load_verify_locations (*ctx, NULL, "/etc/ssl/certs"))
     {
       PRINTF (ERR_HANDLE, _("error loading default verify locations: %s\n"),
-	       ERR_error_string (ERR_get_error (), NULL));
+	      ERR_error_string (ERR_get_error (), NULL));
       if (reqcert > 1)
 	return E_SSL_FAILURE;
     }
@@ -602,7 +951,7 @@ start_ssl (HANDLE int sock, int reqcert, int verbose,
   if (!SSL_CTX_set_default_verify_paths (*ctx))
     {
       PRINTF (ERR_HANDLE, _("error setting default verify path: %s\n"),
-	       ERR_error_string (ERR_get_error (), NULL));
+	      ERR_error_string (ERR_get_error (), NULL));
       if (reqcert > 1)
 	return E_SSL_FAILURE;
     }
@@ -632,8 +981,8 @@ start_ssl (HANDLE int sock, int reqcert, int verbose,
       verify_result = SSL_get_verify_result (*ssl);
 
       /* Following two steps are optional and not required for
-	 data exchange to be successful except the client couldn't verfiy
-	 the server certificate.  */
+         data exchange to be successful except the client couldn't verfiy
+         the server certificate.  */
       if (verify_result || verbose)
 	{
 	  /* Get the cipher.  */
@@ -648,24 +997,24 @@ start_ssl (HANDLE int sock, int reqcert, int verbose,
 	    }
 	  else
 	    {
-	      PRINTF (STD_HANDLE, _("Server certificate:\n"));
+	      PRINTF (STD_HANDLE, _("Server certificate info:\n"));
 
 	      str = X509_NAME_oneline (X509_get_subject_name (server_cert),
 				       0, 0);
 	      if (str)
 		{
-		  PRINTF (STD_HANDLE, _("  subject: %s\n"), str);
+		  PRINTF (STD_HANDLE, _("  DN: %s\n"), str);
 		  free (str);
 		}
 	      str = X509_NAME_oneline (X509_get_issuer_name (server_cert),
 				       0, 0);
 	      if (str)
 		{
-		  PRINTF (STD_HANDLE, _("  issuer: %s\n"), str);
+		  PRINTF (STD_HANDLE, _("  Issuer's DN: %s\n"), str);
 		  free (str);
 		}
 	      /* We could do all sorts of certificate verification stuff
-		 here before deallocating the certificate.  */
+	         here before deallocating the certificate.  */
 
 	      PRINTF (STD_HANDLE, "\n");
 	    }
@@ -674,7 +1023,7 @@ start_ssl (HANDLE int sock, int reqcert, int verbose,
       if ((verify_result = SSL_get_verify_result (*ssl)) != X509_V_OK)
 	{
 	  PRINTF (ERR_HANDLE, "Server certificate is not ok: %s!\n",
-		   X509_verify_cert_error_string (verify_result));
+		  X509_verify_cert_error_string (verify_result));
 	  if (reqcert >= 2)
 	    return E_SSL_FAILURE;
 	}
@@ -684,3 +1033,4 @@ start_ssl (HANDLE int sock, int reqcert, int verbose,
 
   return 0;
 }
+#endif
