@@ -106,7 +106,7 @@ static void
 print_usage (FILE *stream, const char *program)
 {
 #ifdef USE_SLP
-  fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n       [--slp [--slp-timeout timeout] [--slp-descr description]\n"),
+  fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n       [--slp [--slp-timeout timeout] [--slp-descr description]]\n"),
 	   program);
 #else
   fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n"),
@@ -873,7 +873,8 @@ server_run (const char *certificate, const char *privatekey)
 	  if (pollfd_conn[i].revents & (POLLRDNORM|POLLERR|POLLHUP|POLLNVAL))
 	    {
 	      /* Accept the connection.  */
-	      int fd, ssl_err;
+	      int ssl_err;
+	      long fd;
 	      request_header req;
 	      char buf[256];
 	      char *locale, *username;
