@@ -66,7 +66,7 @@ print_help (const char *program)
   print_usage (stdout, program);
   fprintf (stdout, _("%s - create a new user\n\n"), program);
 
-  fputs (_("  -c comment     Set the gecos field for the new account\n"),
+  fputs (_("  -c comment     Set the GECOS field for the new account\n"),
 	 stdout);
   fputs (_(" --show-defaults Print default values\n"), stdout);
   fputs (_(" --save-defaults Save modified default values\n"), stdout);
@@ -134,8 +134,8 @@ static const char *short_options_all = "c:D:d:e:f:G:g:k:MmoP:p:rs:u:v";
 
 static struct option long_options_D[] = {
   {"home", required_argument, NULL, 'b'},
-#if 0 /* def  USE_LDAP XXX */
-  {"binddn", required_argument, NULL, 'D'},
+#ifdef  USE_LDAP
+  {"binddn", required_argument, NULL, '\250'},
 #endif
   {"home", required_argument, NULL, 'd'},
   {"expire", required_argument, NULL, 'e'},
@@ -1018,6 +1018,7 @@ main (int argc, char **argv)
 	    }
 	  new_comment = optarg;
 	  break;
+	case '\250':
         case 'D':
 #ifdef USE_LDAP
 	  if (!modify_defaults)
